      program corpegami

      implicit none

      integer nxmax, nymax, niters
      parameter (nxmax = 1000)
      parameter (nymax = 1000)
      parameter (niters = 100)

      real xold(0:nxmax+1, 0:nymax+1), xnew(0:nxmax+1, 0:nymax+1)
      real b(0:nxmax+1, 0:nymax+1)
      real btmp(nxmax,nymax)

      double precision res
      real secnds, start, time 

      character*32 filename     ! Increase size if longer file name is used

      integer i, j, nx, ny, iloop

c     
c     Read data file into temporary storage
c     

      filename = 'regor.dat'

      call datread(filename, btmp, nxmax, nymax, nx, ny)

      if (nx .eq. 0 .or. ny .eq. 0) stop

      write(*,*) 'Problem size is ', nx, ' x ', ny 

c
c     Initialise b to zero 
c

      do i = 0, nxmax + 1
         do j = 0, nymax + 1
            b(i,j) = 0
         end do
      end do

c     
c     Copy input data into array with halos
c     

      do i = 1, nx
         do j = 1, ny
            b(i,j) = btmp(i,j)
         end do
      end do
c     
c     Initialise solution xnew
c     

      do i = 0, nxmax + 1
         do j = 0, nymax + 1
            xnew(i,j) = b(i,j)
         end do
      end do

      call residue(res, xnew, b, nxmax, nymax, nx, ny)
      write(*,*) 'Initial residue = ', res
      write(*,*)

      start = secnds(0.0) 

      do iloop = 1, niters

         call new2old(xnew, xold, nxmax, nymax, nx, ny)

         call jacobistep(xnew, xold, b, nxmax, nymax, nx, ny)

         call residue(res, xnew, b, nxmax, nymax, nx, ny)

c         write(*,*) 'iteration ', iloop, ', residue = ', res
         
      end do
      
      time = secnds(start)
      
      write(*,*) 'Final residue = ', res
      write(*,*)
      write(*,*) 'Time  = ',time  

      call pgmwrite('finalimage.pgm', xnew, nxmax, nymax, nx, ny)

      end 

c
c     Subroutines and functions below
c

      subroutine new2old(xnew, xold, nxmax, nymax, nx, ny)

      implicit none

      integer nxmax, nymax, nx, ny

      real xnew(0:nxmax+1, 0:nymax+1), xold(0:nxmax+1, 0:nymax+1)

      integer i, j

      do j = 1, ny
         do i = 1, nx

            xold(i,j) = xnew(i,j) 

         end do
      end do

      return
      end 


      subroutine jacobistep(xnew, xold, b, nxmax, nymax, nx, ny)

      implicit none

      integer nxmax, nymax, nx, ny
      real xnew(0:nxmax+1, 0:nymax+1), xold(0:nxmax+1, 0:nymax+1)
      real b(0:nxmax+1, 0:nymax+1) 

      integer i, j


      do j = 1, ny
         do i = 1, nx

            xnew(i,j) = 0.25*(xold(i+1,j)+xold(i-1,j)+xold(i,j+1)+
     &           xold(i,j-1) - b(i,j))
         end do
      end do

      return
      end 


      subroutine residue(res, x, b, nxmax, nymax, nx, ny)

      implicit none

      integer nxmax, nymax, nx, ny
      real x(0:nxmax+1, 0:nymax+1), b(0:nxmax+1, 0:nymax+1)

      double precision res, tmp

      integer i, j

      res = 0.0

      do i = 1, nx
         do j = 1, ny

            tmp = x(i+1,j)+x(i-1,j)+x(i,j+1)+x(i,j-1) - 4.0*x(i,j) - 
     &           b(i,j)
            res = res + tmp*tmp

         end do
      end do

      res = sqrt(res)

      return
      end


      subroutine datread(filename, pixmap, nxmax, nymax, nx, ny)

      implicit none

      character*32 filename
      integer nxmax, nymax, nx, ny
      real pixmap(nxmax, nymax) 

      integer i, j, iounit

      parameter (iounit = 12)

      open(unit=iounit, file=filename)

      read(iounit,*) nx, ny

      if (nx .gt. nxmax .or. ny .gt. nymax) then
         write(*,*) 'Not enough space in pgmread'
         nx = 0
         ny = 0
         return
      end if

      read(iounit,*) ((pixmap(i,j), i=1,nx), j=1,ny)

      close(unit=iounit)

      return
      end


      subroutine pgmwrite(filename, x, nxmax, nymax, nx, ny)

      implicit none

      character*(*) filename
      integer nxmax, nymax, nx, ny
      real x(0:nxmax+1, 0:nymax+1)

      real maxval_f77, minval_f77
      real xmax, xmin

      integer rgb(nx, ny)
      integer iounit
      parameter (iounit = 10)

      integer i, j

      xmax = maxval_f77(x, nxmax, nymax, nx, ny)
      xmin = minval_f77(x, nxmax, nymax, nx, ny)


      if (xmin .lt. 0 .or. xmax .gt. 255.0) then

         do i = 1, nx
            do j = 1, ny
               rgb(i,j) = int((255.0 * ((x(i,j)-xmin)/(xmax-xmin))) 
     &              + 0.5)
            end do
         end do

      else
         
         do i = 1, nx
            do j = 1,ny
               rgb(i,j) = int(x(i,j) + 0.5)
            end do
         end do

      end if

      open(unit=iounit, file=filename)

      write(iounit,fmt='(''P2''/''# Written by pgmwrite'')')
      write(iounit,*) nx, ny
      write(iounit,*) 255
      write(iounit,fmt='(17(i3,'' ''))') ((rgb(i,j), i=1,nx), j=1,ny)  

      close(unit=iounit)

      return
      end 


      real function maxval_f77(tmp, nxmax, nymax, nx, ny)

      implicit none

      integer nxmax, nymax, nx, ny
      real tmp(0:nxmax+1,0:nymax+1)

      integer i, j 
      real tmpmax
      
      tmpmax = -1.0E-32

      do i = 1, nx
         do j = 1, ny         
            if (tmp(i,j) .ge. tmpmax) then
               tmpmax = tmp(i,j)
            end if
         end do
      end do

      maxval_f77 = tmpmax

      end


      real function minval_f77(tmp, nxmax, nymax, nx, ny)
 
      implicit none

      integer nxmax, nymax, nx, ny
      real tmp(0:nxmax+1,0:nymax+1)

      integer i, j 
      real tmpmin
      
      tmpmin = 1.0E+32

      do i = 1, nx
         do j = 1, ny         
            if (tmp(i,j) .lt. tmpmin) then
               tmpmin = tmp(i,j)
            end if
         end do
      end do

      minval_f77 = tmpmin

      end
