program corpegami

  implicit none

  integer, parameter :: dp = kind(1.0d0)
  integer, parameter :: nxmax = 1000, nymax = 1000, niters = 100

  real, dimension(0:nxmax+1,0:nymax+1) :: xold, xnew, b
  real, dimension(nxmax,nymax) :: btmp

  real (kind = dp) :: res
  real :: secnds, start, time 

  integer, parameter :: maxlen = 32

  character*(maxlen) :: filename

  integer :: i, j, nx, ny, xmax, xmin, iloop

!
! Read data file into temporary storage
!

  filename = 'regor.dat'

  call datread(filename, btmp, nxmax, nymax, nx, ny)

  if (nx == 0 .or. ny == 0) stop

  write(*,*) 'Problem size is ', nx, ' x ', ny 

!
! Copy into array with halos
!

  b(:,:) = 0
  b(1:nx,1:ny) = btmp(1:nx,1:ny)

!
! Initialise solution xnew
!

  xnew(:,:) = b(:,:)


  call residue(res, xnew, b, nxmax, nymax, nx, ny)
  write(*,*) 'Initial residue = ', res
  write(*,*)

  start = secnds(0.0) 

  do iloop = 1, niters

    call new2old(xnew, xold, nxmax, nymax, nx, ny)

    call jacobistep(xnew, xold, b, nxmax, nymax, nx, ny)

    call residue(res, xnew, b, nxmax, nymax, nx, ny)

    !write(*,*) 'iteration ', iloop, ', residue = ', res
    
   end do
 
   time = secnds(start)
 
   write(*,*) 'Final residue = ', res
   write(*,*)
   write(*,*) 'Time  = ',time  

   call pgmwrite('finalimage.pgm', xnew, nxmax, nymax, nx, ny)

end program corpegami

subroutine new2old(xnew, xold, nxmax, nymax, nx, ny)

  implicit none

  integer :: nxmax, nymax, nx, ny
  real, dimension(0:nxmax+1, 0:nymax+1) :: xnew, xold

  integer :: i, j


  do j = 1, ny
    do i = 1, nx

      xold(i,j) = xnew(i,j) 

    end do
  end do

end subroutine new2old

subroutine jacobistep(xnew, xold, b, nxmax, nymax, nx, ny)

  implicit none

  integer :: nxmax, nymax, nx, ny
  real, dimension(0:nxmax+1, 0:nymax+1) :: xnew, xold, b

  integer :: i, j


  do j = 1, ny
    do i = 1, nx

      xnew(i,j) = 0.25*(xold(i+1,j)+xold(i-1,j)+xold(i,j+1)+xold(i,j-1) &
                         - b(i,j))
    end do
  end do

end subroutine jacobistep


subroutine residue(res, x, b, nxmax, nymax, nx, ny)

  implicit none

  integer, parameter :: dp = kind(1.0d0)

  integer :: nxmax, nymax, nx, ny
  real, dimension(0:nxmax+1, 0:nymax+1) :: x, b

  real (kind = dp) :: res, tmp

  integer :: i, j

  res = 0.0

  do i = 1, nx
    do j = 1, ny

      tmp = x(i+1,j)+x(i-1,j)+x(i,j+1)+x(i,j-1) - 4.0*x(i,j) - b(i,j)
      res = res + tmp*tmp

    end do
  end do

  res = sqrt(res)

end subroutine residue


subroutine datread(filename, pixmap, nxmax, nymax, nx, ny)

  implicit none

  character*(*) :: filename
  integer :: nxmax, nymax, nx, ny
  real, dimension(nxmax, nymax) :: pixmap

  integer i, j

  integer, parameter :: iounit = 12

  open(unit=iounit, file=filename)

  read(iounit,*) nx, ny

  if (nx > nxmax .or. ny > nymax) then
    write(*,*) 'Not enough space in pgmread'
    nx = 0
    ny = 0
    return
  end if

  read(iounit,*) ((pixmap(i,j), i=1,nx), j=1,ny)

  close(unit=iounit)

end subroutine datread


subroutine pgmwrite(filename, x, nxmax, nymax, nx, ny)

  implicit none

  character*(*) :: filename
  integer :: nxmax, nymax, nx, ny
  real, dimension(0:nxmax+1, 0:nymax+1) :: x

  real :: xmax, xmin

  integer, dimension(nx, ny) :: rgb
  integer, parameter :: iounit = 10

  integer :: i, j

  xmax = maxval(x(1:nx, 1:ny))
  xmin = minval(x(1:nx, 1:ny))

  if (xmin < 0 .or. xmax > 255.0) then

    rgb(1:nx,1:ny) = int((255.0 * ((x(1:nx, 1:ny)-xmin)/(xmax-xmin))) + 0.5)

  else

    rgb(1:nx,1:ny) = int(x(1:nx, 1:ny) + 0.5)

  end if

  open(unit=iounit, file=filename)

  write(iounit,fmt='(''P2''/''# Written by pgmwrite'')')
  write(iounit,*) nx, ny
  write(iounit,*) 255
  write(iounit,fmt='(17(i3,'' ''))') ((rgb(i,j), i=1,nx), j=1,ny)  

  close(unit=iounit)

end subroutine pgmwrite
