program area_mandelbrot
  implicit none
!
  integer, parameter :: sp = kind(1.0)   
  integer, parameter :: dp = kind(1.0d0)
  integer :: i, j, numinside
  integer (kind=sp) :: iseed
  integer, parameter :: npoints = 1000, maxiter = 10000
  real (kind=sp) :: ran
  real (kind=dp) :: r1, r2, area, error
  complex (kind=dp) :: c(npoints), z
!
! 1) Generate npoints random points in the complex plane.
!
  iseed = 54321
  r1=ran(iseed)
  do i=1,npoints
     r1=ran(iseed)
     r2=ran(iseed)
     c(i)=cmplx(-2.0+2.5*r1,1.125*r2)
  end do


!
! 2) Calculate area of mandelbrot set by monte carlo sampling :
!
!    2a) Outer loop runs over npoints, initialize z=c
!
!    2b) Inner loop has the iteration z=z*z+c, and threshold test
!

!
! 3) Output results
!
  area = 2.0*2.5*1.125 * real(numinside)/real(npoints)
  error = area/sqrt(real(npoints))
  print *, "Area of Mandelbrot set = ",area," +/- ",error
!
  stop
end program area_mandelbrot
